//
//  DXToolbarFactory.m
//  DiceX
//
//  Created by Andrew Merenbach on 7/15/07.
//  Copyright 2007 Andrew Merenbach. All rights reserved.
//

#import "DXToolbarFactory.h"


static void addToolbarItemToFactory(NSMutableDictionary *theDict,
	NSString *identifier, NSString *label, NSString *paletteLabel, NSString *toolTip,
	id target, NSImage *image, SEL action, NSMenu *menu)
{
	NSMenuItem *mItem;
	// here we create the NSToolbarItem and setup its attributes in line with the parameters
	NSToolbarItem *item = [[NSToolbarItem alloc] initWithItemIdentifier:identifier];
	[item setLabel:label];
	if (paletteLabel) {
		[item setPaletteLabel:paletteLabel];
	} else {
		[item setPaletteLabel:label];
	}
	[item setToolTip:toolTip];
	
	[item setTarget:target];
	// the settingSelector parameter can either be @selector(setView:) or @selector(setImage:).  Pass in the right
	// one depending upon whether your NSToolbarItem will have a custom view or an image, respectively
	// (in the itemContent parameter).  Then this next line will do the right thing automatically.
	//[item performSelector:settingSelector withObject:itemContent];
	[item setImage:image];
	
	[item setAction:action];
	
	// If this NSToolbarItem is supposed to have a menu "form representation" associated with it (for text-only mode), 
	// we set it up here.  Actually, you have to hand an NSMenuItem (not a complete NSMenu) to the toolbar item, 
	// so we create a dummy NSMenuItem that has our real menu as a submenu.
	if (menu) {
		//NSLog(@"addToolbarItem(): menu title = %@", [menu title]);
	// we actually need an NSMenuItem here, so we construct one
		mItem = [[NSMenuItem alloc] init];
		[mItem setSubmenu:menu];
		[mItem setTitle:[menu title]];
		[item setMenuFormRepresentation:mItem];
		[mItem release];
	}
	// Now that we've setup all the settings for this new toolbar item, we add it to the dictionary.
	// The dictionary retains the toolbar item for us, which is why we could autorelease it when we created
	// it (above).
	[theDict setObject:item forKey:identifier];
	[item release];
}


@implementation DXToolbarFactory

- (id)init {
	if (nil != (self = [super init])) {
		delegate = nil;
		toolbarItems = [[NSMutableDictionary allocWithZone:[self zone]] init];
	}
	return self;
}

- (void)dealloc {
	[toolbarItems release];
	toolbarItems = nil;
	
	[super dealloc];
}

- (void)setupToolbarWithIdentifier:(NSString *)identifier {
	if (identifier) {
		NSToolbar *toolbar = [[NSToolbar alloc] initWithIdentifier:identifier];
		[toolbar setDelegate:self];
		
		NSBundle *bundle = [NSBundle mainBundle];	// [bundleforclass:selfclass?]
		NSString *bundleIdentifier = [bundle bundleIdentifier];
		NSString *filename = [bundleIdentifier stringByAppendingFormat:@".%@", identifier];
		NSString *path = [bundle pathForResource:filename ofType:@"plist"];
		NSDictionary *toolbarDict = [[NSDictionary alloc] initWithContentsOfFile:path];
		
		{
			NSDictionary *options = [toolbarDict objectForKey:@"Options"];
			NSNumber *allowsUserCustomizationValue = [options objectForKey:@"AllowsUserCustomization"];
			NSNumber *autosavesConfigurationValue = [options objectForKey:@"AutosavesConfiguration"];
			NSNumber *displayModeValue = [options objectForKey:@"DisplayMode"];
			NSNumber *sizeModeValue = [options objectForKey:@"SizeMode"];
			
			BOOL allowsUserCustomization = (allowsUserCustomizationValue ? [allowsUserCustomizationValue boolValue] : NO);
			BOOL autosavesConfiguration = (autosavesConfigurationValue ? [autosavesConfigurationValue boolValue] : NO);
			NSToolbarDisplayMode displayMode = (NSToolbarDisplayMode)[displayModeValue intValue];
			NSToolbarSizeMode sizeMode = (NSToolbarSizeMode)[sizeModeValue intValue];
			
			[toolbar setAllowsUserCustomization:allowsUserCustomization];
			[toolbar setAutosavesConfiguration:autosavesConfiguration];
			[toolbar setDisplayMode:displayMode];
			[toolbar setSizeMode:sizeMode];
		}
		
		NSArray *defaultItems = [toolbarDict objectForKey:@"DefaultItems"];
		[self setDefaultItemIdentifiers:defaultItems];
		
		NSArray *allowedItems = [toolbarDict objectForKey:@"AllowedItems"];
		[self setAllowedItemIdentifiers:allowedItems];
		
		NSArray *selectableItems = [toolbarDict objectForKey:@"SelectableItems"];
		[self setSelectableItemIdentifiers:selectableItems];
		
		NSArray *toolbarItemsTemp = [toolbarDict objectForKey:@"Items"];
		NSEnumerator *e = [toolbarItemsTemp objectEnumerator];
		NSDictionary *itemDict;

		while (itemDict = [e nextObject]) {
			addToolbarItemToFactory(toolbarItems,
				[itemDict objectForKey:@"Identifier"],
				NSLocalizedStringFromTable([itemDict objectForKey:@"Label"], @"ToolbarItems", @"Dummy Label"),
				NSLocalizedStringFromTable([itemDict objectForKey:@"PaletteLabel"], @"ToolbarItems", @"Dummy Palette Label"),
				NSLocalizedStringFromTable([itemDict objectForKey:@"ToolTip"], @"ToolbarItems", @"Dummy ToolTip"),
				delegate,
				//[itemDict objectForKey:@"SettingSelector"],
				[NSImage imageNamed:[itemDict objectForKey:@"Image"]],
				NSSelectorFromString([itemDict objectForKey:@"Action"]),
				nil
			);
		}
		
		[window setToolbar:toolbar];
		[toolbar release];
	}
}

// This method is required of NSToolbar delegates.  It takes an identifier, and returns the matching NSToolbarItem.
// It also takes a parameter telling whether this toolbar item is going into an actual toolbar, or whether it's
// going to be displayed in a customization palette.
- (NSToolbarItem *)toolbar:(NSToolbar *)toolbar itemForItemIdentifier:(NSString *)itemIdentifier willBeInsertedIntoToolbar:(BOOL)flag
{
	// We create and autorelease a new NSToolbarItem, and then go through the process of setting up its
	// attributes from the master toolbar item matching that identifier in our dictionary of items.
	NSToolbarItem *newItem = [[[NSToolbarItem alloc] initWithItemIdentifier:itemIdentifier] autorelease];
	//NSToolbarItem *item = [toolbarItems objectForKey:itemIdentifier];
	NSToolbarItem *item = [toolbarItems objectForKey:itemIdentifier];
	
	[newItem setLabel:[item label]];
	[newItem setPaletteLabel:[item paletteLabel]];
	
	if (nil != [item view]) {
		[newItem setView:[item view]];
	} else {
		[newItem setImage:[item image]];
	}
	
	[newItem setToolTip:[item toolTip]];
	[newItem setTarget:[item target]];
	[newItem setAction:[item action]];
	[newItem setMenuFormRepresentation:[item menuFormRepresentation]];
	
	// If we have a custom view, we *have* to set the min/max size - otherwise, it'll default to 0, 0 and the custom
	// view won't show up at all!  This doesn't affect toolbar items with images, however.
	if (nil != [newItem view]) {
		[newItem setMinSize:[[item view] bounds].size];
		[newItem setMaxSize:[[item view] bounds].size];
	}

	return newItem;
}

- (NSArray *)toolbarDefaultItemIdentifiers:(NSToolbar *)toolbar {
	return [[AM_toolbarDefaultItemIdentifiers retain] autorelease];
}

- (NSArray *)toolbarAllowedItemIdentifiers:(NSToolbar *)toolbar {
	return [[AM_toolbarAllowedItemIdentifiers retain] autorelease];
}

- (NSArray *)toolbarSelectableItemIdentifiers:(NSToolbar *)toolbar {
	return [[AM_toolbarSelectableItemIdentifiers retain] autorelease];
}

@end

@implementation DXToolbarFactory (Accessors)

- (NSArray *)defaultItemIdentifiers {
	return [[AM_toolbarDefaultItemIdentifiers retain] autorelease];
}

- (void)setDefaultItemIdentifiers:(NSArray *)anArray {
	if (anArray != AM_toolbarDefaultItemIdentifiers) {
		[AM_toolbarDefaultItemIdentifiers release];
		AM_toolbarDefaultItemIdentifiers = [anArray retain];
	}
}

- (NSArray *)allowedItemIdentifiers {
	return [[AM_toolbarAllowedItemIdentifiers retain] autorelease];
}

- (void)setAllowedItemIdentifiers:(NSArray *)anArray {
	if (anArray != AM_toolbarAllowedItemIdentifiers) {
		[AM_toolbarAllowedItemIdentifiers release];
		AM_toolbarAllowedItemIdentifiers = [anArray retain];
	}
}

- (NSArray *)selectableItemIdentifiers {
	return [[AM_toolbarSelectableItemIdentifiers retain] autorelease];
}

- (void)setSelectableItemIdentifiers:(NSArray *)anArray {
	if (anArray != AM_toolbarSelectableItemIdentifiers) {
		[AM_toolbarSelectableItemIdentifiers release];
		AM_toolbarSelectableItemIdentifiers = [anArray retain];
	}
}

/*- (id)delegate {
	return delegate;
}

- (void)setDelegate:(id)anObject {
	delegate = anObject;
}*/

@end
